﻿using System;
using System.Collections.Generic;

namespace HeuristicLab.Problems.ProgramSynthesis.Push.Configuration {
  using HeuristicLab.Core;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Expressions;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  public class EnabledExpressionsChangedEventArgs : EventArgs {
    public IEnumerable<string> AddedExpressions { get; private set; }
    public IEnumerable<string> RemovedExpressions { get; private set; }

    public EnabledExpressionsChangedEventArgs(IEnumerable<string> addedExpressions, IEnumerable<string> removedExpressions) {
      AddedExpressions = addedExpressions;
      RemovedExpressions = removedExpressions;
    }
  }

  public interface IEnabledExpressionsConfiguration : INamedItem {
    event EventHandler<EnabledExpressionsChangedEventArgs> EnabledExpressionsChanged;

    IList<string> EnabledExpressions { get; }

    void EnableStack(StackType type);
    void DisableStack(StackType type);
    void EnableExpressionOfStack(StackType type);
    void DisableExpressionOfStack(StackType type);
    void EnableExpression(string name);
    void DisableExpression(string name);
    void EnableExpression<T>() where T : Expression;
    void DisableExpression<T>() where T : Expression;
  }
}
