﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  /// <summary>
  ///     Duplicates the top item on the INTEGER stack. Does not pop its argument
  ///     (which, if it did, would negate the effect of the duplication!).
  /// </summary>
  /// <typeparam name="T">Stacktype</typeparam>
  public abstract class DuplicateExpression<T> : StatelessExpression {
    protected void Eval(IStack<T> stack) {
      if (stack.Count == 0) return;

      stack.Push(stack.Top);
    }

    protected void Eval(IStack<Expression> stack) {
      if (stack.Count == 0) return;

      var top = stack.Top;

      stack.Push(top);
    }
  }

  [PushExpression(StackType.Integer, "INTEGER.DUP")]
  public class IntegerDuplicateExpression : DuplicateExpression<long> {
    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.IntegerStack);
    }
  }

  [PushExpression(StackType.Float, "FLOAT.DUP")]
  public class FloatDuplicateExpression : DuplicateExpression<double> {
    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.FloatStack);
    }
  }

  [PushExpression(StackType.Boolean, "BOOLEAN.DUP")]
  public class BooleanDuplicateExpression : DuplicateExpression<bool> {
    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.BooleanStack);
    }
  }

  [PushExpression(StackType.Name, "NAME.DUP")]
  public class NameDuplicateExpression : DuplicateExpression<string> {
    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.NameStack);
    }
  }

  [PushExpression(StackType.Exec, "EXEC.DUP")]
  public class ExecDuplicateExpression : DuplicateExpression<Expression> {
    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.ExecStack);
    }
  }

  [PushExpression(StackType.Code, "CODE.DUP")]
  public class CodeDuplicateExpression : DuplicateExpression<Expression> {
    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.CodeStack);
    }
  }
}