﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;

  using Interpreter;
  using Stack;

  /// <summary>
  ///     Empties the given stack.
  /// </summary>
  /// <typeparam name="T">Stacktype</typeparam>
  public abstract class FlushExpression<T> : StatelessExpression {
    public void Eval(IStack<T> stack) {
      stack.Clear();
    }
  }

  [PushExpression(StackType.Integer, "INTEGER.FLUSH")]
  public class IntegerFlushExpression : FlushExpression<long> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.IntegerStack);
    }
  }

  [PushExpression(StackType.Float, "FLOAT.FLUSH")]
  public class FloatFlushExpression : FlushExpression<double> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.FloatStack);
    }
  }

  [PushExpression(StackType.Boolean, "BOOLEAN.FLUSH")]
  public class BooleanFlushExpression : FlushExpression<bool> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.BooleanStack);
    }
  }

  [PushExpression(StackType.Name, "NAME.FLUSH")]
  public class NameFlushExpression : FlushExpression<string> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.NameStack);
    }
  }

  [PushExpression(StackType.Exec, "EXEC.FLUSH")]
  public class ExecFlushExpression : FlushExpression<Expression> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.ExecStack);
    }
  }

  [PushExpression(StackType.Code, "CODE.FLUSH")]
  public class CodeFlushExpression : FlushExpression<Expression> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.CodeStack);
    }
  }
}