﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using System.Linq;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;
  using Interpreter;

  public class NameDefineXExecExpression : StatefullExpression<string> {
    public NameDefineXExecExpression(string state) : base(state) { }

    public override void Eval(IPushGpInterpreter interpreter) {
      Expression expression;
      if (!interpreter.IsNameQuoteFlagSet &&
          interpreter.CustomExpressions.TryGetValue(State, out expression)) {
        interpreter.ExecStack.Push(expression);
      } else {
        interpreter.NameStack.Push(State);
        interpreter.IsNameQuoteFlagSet = false;
      }
    }

    public override string StringRepresentation { get { return this.State; } }
  }

  /// <summary>
  ///     Sets a flag indicating that the next State encountered will be pushed onto the NAME stack (and not have its
  ///     associated value
  ///     pushed onto the EXEC stack), regardless of whether or not it has a definition. Upon encountering such a State and
  ///     pushing it
  ///     onto the NAME stack the flag will be cleared (whether or not the pushed State had a definition).
  /// </summary>
  [PushExpression(StackType.Name, "NAME.QUOTE")]
  public class NameQuoteExpression : StatelessExpression {
    public override void Eval(IPushGpInterpreter interpreter) {
      interpreter.IsNameQuoteFlagSet = true;
    }
  }

  /// <summary>
  ///     Pushes a randomly selected NAME that already has a definition.
  /// </summary>
  [PushExpression(StackType.Name, "NAME.RANDBOUNDNAME")]
  public class NameRandBoundNameExpression : StatelessExpression {
    public override void Eval(IPushGpInterpreter interpreter) {
      if (interpreter.CustomExpressions.Count == 0)
        return;

      var index = interpreter.CustomExpressions.Keys.Count == 1
        ? 0
        : interpreter.Random.Next(0, interpreter.CustomExpressions.Keys.Count - 1);

      var state = interpreter.CustomExpressions.Keys.ElementAt(index);

      interpreter.NameStack.Push(state);
    }
  }
}