﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;

  using Interpreter;
  using Stack;

  public abstract class PopExpression<T> : StatelessExpression {
    public void Eval(IStack<T> stack) {
      if (stack.Count == 0) return;

      stack.Pop();
    }
  }

  [PushExpression(StackType.Integer, "INTEGER.POP")]
  public class IntegerPopExpression : PopExpression<long> {
    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.IntegerStack);
    }
  }

  [PushExpression(StackType.Float, "FLOAT.POP")]
  public class FloatPopExpression : PopExpression<double> {
    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.FloatStack);
    }
  }

  [PushExpression(StackType.Boolean, "BOOLEAN.POP")]
  public class BooleanPopExpression : PopExpression<bool> {
    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.BooleanStack);
    }
  }

  [PushExpression(StackType.Name, "NAME.POP")]
  public class NamePopExpression : PopExpression<string> {
    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.NameStack);
    }
  }

  [PushExpression(StackType.Exec, "EXEC.POP")]
  public class ExecPopExpression : PopExpression<Expression> {
    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.ExecStack);
    }
  }

  [PushExpression(StackType.Code, "CODE.POP")]
  public class CodePopExpression : PopExpression<Expression> {
    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.CodeStack);
    }
  }
}