﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;

  using Interpreter;
  using Stack;

  public abstract class RotateExpression<T> : StatelessExpression {
    public void Eval(IStack<T> stack) {
      if (stack.Count < 3) return;

      stack.Swap(3);
    }
  }


  [PushExpression(StackType.Integer, "INTEGER.ROT")]
  public class IntegerRotateExpression : RotateExpression<long> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.IntegerStack);
    }
  }

  [PushExpression(StackType.Float, "FLOAT.ROT")]
  public class FloatRotateExpression : RotateExpression<double> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.FloatStack);
    }
  }

  [PushExpression(StackType.Boolean, "BOOLEAN.ROT")]
  public class BooleanRotateExpression : RotateExpression<bool> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.BooleanStack);
    }
  }

  [PushExpression(StackType.Name, "NAME.ROT")]
  public class NameRotateExpression : RotateExpression<string> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.NameStack);
    }
  }

  [PushExpression(StackType.Exec, "EXEC.ROT")]
  public class ExecRotateExpression : RotateExpression<Expression> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.ExecStack);
    }
  }

  [PushExpression(StackType.Code, "CODE.ROT")]
  public class CodeRotateExpression : RotateExpression<Expression> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.CodeStack);
    }
  }
}