﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;

  using Interpreter;
  using Stack;

  public abstract class ShoveExpression<T> : StatelessExpression {
    protected void Eval(IStack<T> stack, IStack<long> integerStack, int count = 1) {
      if (integerStack.Count == 0 ||
          integerStack.Top > stack.Count - count ||
          integerStack.Top < 0 ||
          stack.Count < 2)
        return;

      var index = (int)integerStack.Pop();
      var item = stack.Pop();

      stack.Insert(index, item);
    }
  }

  [PushExpression(StackType.Integer, "INTEGER.SHOVE")]
  public class IntegerShoveExpression : ShoveExpression<long> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.IntegerStack, interpreter.IntegerStack, 2);
    }
  }

  [PushExpression(StackType.Float, "FLOAT.SHOVE")]
  public class FloatShoveExpression : ShoveExpression<double> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.FloatStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackType.Boolean, "BOOLEAN.SHOVE")]
  public class BooleanShoveExpression : ShoveExpression<bool> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.BooleanStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackType.Name, "NAME.SHOVE")]
  public class NameShoveExpression : ShoveExpression<string> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.NameStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackType.Exec, "EXEC.SHOVE")]
  public class ExecShoveExpression : ShoveExpression<Expression> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.ExecStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackType.Code, "CODE.SHOVE")]
  public class CodeShoveExpression : ShoveExpression<Expression> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.CodeStack, interpreter.IntegerStack);
    }
  }
}