﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;

  using Interpreter;
  using Stack;

  /// <summary>
  ///     Pushes the stack depth onto the INTEGER stack (thereby increasing it!).
  /// </summary>
  /// <typeparam name="T">Stacktype</typeparam>
  public abstract class StackdepthExpression<T> : StatelessExpression {
    public void Eval(IStack<T> stack, IStack<long> integerStack, bool incremental = false) {
      var count = stack.Count;

      if (incremental) count += 1;

      integerStack.Push(count);
    }
  }

  [PushExpression(StackType.Integer, "INTEGER.STACKDEPTH")]
  public class IntegerStackdepthExpression : StackdepthExpression<long> {

    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.IntegerStack, interpreter.IntegerStack, true);
    }
  }

  [PushExpression(StackType.Float, "FLOAT.STACKDEPTH")]
  public class FloatStackdepthExpression : StackdepthExpression<double> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.FloatStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackType.Boolean, "BOOLEAN.STACKDEPTH")]
  public class BooleanStackdepthExpression : StackdepthExpression<bool> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.BooleanStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackType.Name, "NAME.STACKDEPTH")]
  public class NameStackdepthExpression : StackdepthExpression<string> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.NameStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackType.Exec, "EXEC.STACKDEPTH")]
  public class ExecStackdepthExpression : StackdepthExpression<Expression> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.ExecStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackType.Code, "CODE.STACKDEPTH")]
  public class CodeStackdepthExpression : StackdepthExpression<Expression> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.CodeStack, interpreter.IntegerStack);
    }
  }
}