﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;

  using Interpreter;
  using Stack;

  /// <summary>
  ///     Swaps the top two values.
  /// </summary>
  /// <typeparam name="T">Stacktype</typeparam>
  public abstract class SwapExpression<T> : StatelessExpression {
    public void Eval(IStack<T> stack) {
      if (stack.Count < 2)
        return;

      stack.Swap(2);
    }
  }

  [PushExpression(StackType.Integer, "INTEGER.SWAP")]
  public class IntegerSwapExpression : SwapExpression<long> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.IntegerStack);
    }
  }

  [PushExpression(StackType.Float, "FLOAT.SWAP")]
  public class FloatSwapExpression : SwapExpression<double> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.FloatStack);
    }
  }

  [PushExpression(StackType.Boolean, "BOOLEAN.SWAP")]
  public class BooleanSwapExpression : SwapExpression<bool> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.BooleanStack);
    }
  }

  [PushExpression(StackType.Name, "NAME.SWAP")]
  public class NameSwapExpression : SwapExpression<string> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.NameStack);
    }
  }

  [PushExpression(StackType.Exec, "EXEC.SWAP")]
  public class ExecSwapExpression : SwapExpression<Expression> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.ExecStack);
    }
  }

  [PushExpression(StackType.Code, "CODE.SWAP")]
  public class CodeSwapExpression : SwapExpression<Expression> {
    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.CodeStack);
    }
  }
}