﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter {
  using System;

  using HeuristicLab.Problems.ProgramSynthesis.Push.Data.Pool;
  using HeuristicLab.Core;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Configuration;
  using HeuristicLab.Random;

  public class PushGpInterpreterPool {
    private readonly ObjectPool<PooledPushGpInterpreter> pool;

    public PushGpInterpreterPool(PushGpConfiguration config = null)
      : this(Environment.ProcessorCount * 2, config) {
    }

    public PushGpInterpreterPool(int size, PushGpConfiguration config = null) {
      this.PushGpConfiguration = config ?? new PushGpConfiguration();

      this.pool = new ObjectPool<PooledPushGpInterpreter>(() => new PooledPushGpInterpreter(this, this.PushGpConfiguration), size);
    }

    public PushGpConfiguration PushGpConfiguration { get; private set; }

    public PooledPushGpInterpreter GetInstance(IRandom random = null) {
      var interpreter = this.pool.Allocate();
      interpreter.Random = random ?? new FastRandom();

      return interpreter;
    }

    public void Free(PooledPushGpInterpreter interpreter) {
      this.pool.Free(interpreter);
    }
  }
}
